import React, { useEffect, useState, useRef } from "react"
import { isAdministrator } from "../../services/auth"
import { navigate } from "gatsby"
import { Result, Button, Popconfirm, message, notification, Steps } from "antd"
import {
	CloudUploadOutlined,
	CloudSyncOutlined,
	LoadingOutlined,
	SmileOutlined,
	MehOutlined,
	FrownOutlined,
} from "@ant-design/icons"
import "./Deploy.less"

const { Step } = Steps

const Deploy = () => {
	const [deploying, setDeploying] = useState(false)
	const [status, setStaus] = useState("wait")
	const [errorMsg, setErrorMsg] = useState("")
	const [latest, setLatest] = useState(true)
	let deployTimer = useRef(null)

	const fetchNetlifyBuild = async () => {
		setStaus("wait")
		setDeploying(true)
		clearTimeout(deployTimer.current)
		try {
			const listSiteDeploys = await fetch(
				"https://api.netlify.com/api/v1/sites/" +
					process.env.GATSBY_NETLIFY_SITE_ID +
					"/deploys",
				{
					method: "GET",
					headers: {
						"Content-Type": "application/json",
						Authorization: `Bearer ${process.env.GATSBY_NETLIFY_PERSONAL_ACCESS_TOKEN}`,
					},
				}
			).then((res) => {
				return res.json()
			})
			if (listSiteDeploys) {
				const lastDeploy = listSiteDeploys[0]
				const deployState = lastDeploy.state
				const deployErrorMsg = lastDeploy.error_message
				if (deployState === "ready") {
					setStaus("finish")
					clearTimeout(deployTimer.current)
					setDeploying(false)
				} else {
					if (deployErrorMsg) {
						setStaus("error")
						setErrorMsg(deployErrorMsg)
						clearTimeout(deployTimer.current)
						setDeploying(false)
					} else {
						setStaus("wait")
						deployTimer.current = setTimeout(() => {
							fetchNetlifyBuild()
						}, 60000)
					}
				}
			}
		} catch (error) {
			console.log("Error fetchNetlifyBuild! ", error)
		}
	}
	function netlifyDeploy(e) {
		e.preventDefault()
		if (deploying) {
			message.error("Please wait while site is deploying!")
		} else {
			const data = {
				trigger_branch: "main",
				trigger_title: "Trigger by Backend application",
			}
			fetch(process.env.GATSBY_BUILD_HOOKS, {
				method: "POST",
				headers: {
					"Content-Type": "application/json",
				},
				body: JSON.stringify(data),
			})
			deployTimer.current = setTimeout(() => {
				setLatest(false)
				fetchNetlifyBuild()
			}, 2500)
		}
	}

	useEffect(() => {
		if (!isAdministrator()) {
			notification["warning"]({
				message: "Warning!",
				description: "Your account is not allowed!",
			})
			navigate("/backend/dashboard")
		}
		setLatest(true)
		fetchNetlifyBuild()
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, [])

	return (
		<div>
			<Result
				icon={<CloudUploadOutlined />}
				title="Have you done!, Ready to deploy?"
				extra={
					<Popconfirm
						title="Are you sure to generate website with new data?"
						onConfirm={netlifyDeploy}
						okText="Yes - Go live"
						cancelText="Not yet"
					>
						<a href="#deploy">
							<Button type="primary" loading={deploying} disabled={deploying}>
								Deploy
							</Button>
						</a>
					</Popconfirm>
				}
			/>
			<div className="stepsContainer">
				<Steps size="small" responsive="true">
					<Step
						status={deploying ? `process` : `finish`}
						title={
							deploying
								? `Deploying...`
								: latest
								? `Latest Deploy`
								: `Your Deploy`
						}
						icon={deploying ? <LoadingOutlined /> : <CloudSyncOutlined />}
					/>
					<Step
						status={status}
						title={errorMsg ? `Error!` : status === `finish` ? `Done` : `Wait`}
						description={
							errorMsg
								? errorMsg
								: status === `finish`
								? `Congratulations, website was live.`
								: ``
						}
						icon={
							status === `finish` ? (
								<SmileOutlined />
							) : status === `error` ? (
								<FrownOutlined />
							) : (
								<MehOutlined />
							)
						}
					/>
				</Steps>
			</div>
		</div>
	)
}

export default Deploy
