import React, { useEffect } from "react"
import { isAdministrator } from "../../services/auth"
import * as matter from "gray-matter"
import { Link, navigate } from "gatsby"
import { FormattedMessage } from "react-intl"
import ProTable, { ProColumns } from "@ant-design/pro-table"
import { notification } from "antd"
import { decodeUnicode } from "../../utils/helper"

const columns: ProColumns[] = [
	{
		title: "#",
		dataIndex: "index",
		valueType: "indexBorder",
		width: 48,
		key: "key",
	},
	{
		title: "Title",
		dataIndex: "title",
		filters: true,
	},
	{
		title: "Path",
		dataIndex: "path",
		hideInSearch: true,
	},
	{
		title: "Menu",
		dataIndex: "menu",
		hideInSearch: true,
	},
	{
		title: "Options",
		valueType: "option",
		dataIndex: "path",
		align: "right",
		width: 150,
		fixed: "right",
		render: (text, row, index, action) => [
			<Link key="pageEdit" to={`/backend/pages/${row.path}`}>
				<FormattedMessage id="menu.edit" />
			</Link>,
			<Link key="pageView" to={`/${row.path === "home" ? "" : row.path}`}>
				<FormattedMessage id="menu.view" />
			</Link>,
		],
	},
]

const Pages = () => {
	const handleGitGet = async () => {
		try {
			let appendMainData = []
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/pages/`,
				{
					method: "GET",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})
			if (fetchGitApi && fetchGitApi.length > 0) {
				for (let index = 0; index < fetchGitApi.length; index++) {
					const element = fetchGitApi[index]
					if (element.type === "dir") {
						const dirName = element.name
						const fetchGitApiPost = await fetch(
							`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/pages/${dirName}/index.md`,
							{
								method: "GET",
								headers: {
									authorization:
										"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
									"Content-Type": "application/json; charset=UTF-8",
								},
							}
						)
							.then((response) => {
								return response.json()
							})
							.catch((error) => {
								console.log("Error ", error)
								notification["error"]({
									message: "Error code " + error.status,
									description: error.statusText,
									duration: 0,
								})
							})

						if (fetchGitApiPost) {
							let appendChildData = {}
							appendChildData["path"] = dirName
							appendChildData["sha"] = fetchGitApiPost.sha
							const decodeContent: any = decodeUnicode(fetchGitApiPost.content)
							const markdownData = matter(decodeContent)
							const dataMarkdown = markdownData.data
							if (dataMarkdown) {
								const fields = Object.keys(dataMarkdown)
								fields.map((field) => {
									const fieldValue = dataMarkdown[field]
									if (field === "title") {
										appendChildData["title"] = fieldValue
									} else if (field === "menu") {
										appendChildData["menu"] = fieldValue
									}
									return true
								})
							}
							appendMainData[index] = appendChildData
						}
					}
				}
				return appendMainData
			}
		} catch (error) {
			console.log("Error handleGitGet ", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: JSON.stringify(error),
				duration: 0,
			})
		}
		return false
	}

	const fetchDataSource = async (keyword: string) => {
		return await handleGitGet().then((getFolder) => {
			if (Array.isArray(getFolder)) {
				const allDataSource = getFolder.map((node) => {
					return {
						key: node.sha,
						title: node.title,
						path: node.path,
						menu: node.menu,
					}
				})
				if (keyword) {
					const filterDataSource = allDataSource.filter((item) => {
						return item.title.indexOf(keyword) > -1
					})

					return filterDataSource
				} else {
					return allDataSource
				}
			}
		})
	}

	useEffect(() => {
		if (!isAdministrator()) {
			notification["warning"]({
				message: "Warning!",
				description: "Your account is not allowed!",
			})
			navigate("/backend/dashboard")
		}
	}, [])

	return (
		<ProTable
			columns={columns}
			request={async (filter) => {
				const dataSource = await fetchDataSource(filter.keyword)
				return {
					data: dataSource,
					success: true,
				}
			}}
			rowKey="key"
			options={{
				search: true,
				fullScreen: true,
				reload: false,
				setting: true,
			}}
			toolbar={{
				title: "Pages List.",
				subTitle: "with toolbar search",
				tooltip: "Search filter Title field only!",
			}}
			search={false}
			pagination={{
				pageSize: 10,
				showQuickJumper: false,
				showSizeChanger: true,
			}}
			dateFormatter="string"
		/>
	)
}

export default Pages
