import React, { useEffect, useState } from "react"
import { isAdministrator } from "../../../services/auth"
import { decodeUnicode } from "../../../utils/helper"
import * as matter from "gray-matter"
import { Link, navigate } from "gatsby"
import { Card, Form, Input, Button, notification } from "antd"

const layout = {
	labelCol: { span: 5 },
	wrapperCol: { span: 16 },
}
const tailLayout = {
	wrapperCol: { offset: 5, span: 16 },
}

const EditPage = ({ slug }) => {
	const [sha, setSha] = useState("")
	const [form] = Form.useForm()
	const [dynamicFields, setDynamicFields] = useState([])

	const onFinish = (values) => {
		let contentStr = "---\n"
		if (values) {
			for (const [key, value] of Object.entries(values)) {
				contentStr += key + `: ` + value + `\n`
			}
		}
		contentStr += "---\n\n"
		handleGitUpdate(contentStr, sha).then((updateFile) => {
			if (updateFile) {
				notification["success"]({
					message: "Success!",
					description:
						"Updated markdown/pages/" + slug + "/index.md successfully.",
				})
			} else {
				let errMsg = "No status"
				if (updateFile === 404) {
					errMsg = "Status: 404 Not Found"
				} else if (updateFile === 409) {
					errMsg = "Status: 409 Conflict"
				} else if (updateFile === 422) {
					errMsg = "Status: 422 Unprocessable Entity"
				}
				notification["error"]({
					message: "Error!",
					description: errMsg,
					duration: 0,
				})
			}
		})
	}

	const handleGitGet = async () => {
		try {
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/pages/${slug}/index.md`,
				{
					method: "GET",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return fetchGitApi
			} else {
				notification["info"]({
					message: "Empty Fetch",
				})
			}
		} catch (error) {
			console.log("Error handleGitGet ", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: JSON.stringify(error),
				duration: 0,
			})
		}
		return false
	}
	const handleGitUpdate = async (contentStr, fileSha) => {
		try {
			const data = {
				message:
					"[skip ci] Update markdown/pages/" +
					slug +
					"/index.md from Modern backend.",
				content: btoa(contentStr),
				sha: fileSha,
			}
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/pages/${slug}/index.md`,
				{
					method: "PUT",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
					body: JSON.stringify(data),
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return true
			} else {
				notification["info"]({
					message: "Empty Fetch",
				})
			}
		} catch (error) {
			console.log("Error handleGitUpdate!", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: JSON.stringify(error),
				duration: 0,
			})
		}
		return false
	}

	useEffect(() => {
		if (!isAdministrator()) {
			notification["warning"]({
				message: "Warning!",
				description: "Your account is not allowed!",
			})
			navigate("/backend/dashboard")
		}

		async function fetchOauth() {
			await handleGitGet().then((getFile) => {
				const decodeContent = decodeUnicode(getFile.content)
				const markdownData = matter(decodeContent)
				const dataMarkdown = markdownData.data
				setDynamicFields([dataMarkdown])
				if (getFile !== false) {
					setSha(getFile.sha)
				}
			})
		}
		fetchOauth()
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, [])

	return (
		<Card>
			<Form
				{...layout}
				name="editForm"
				form={form}
				onFinish={onFinish}
				className="customForm"
			>
				{dynamicFields &&
					dynamicFields.map((fields) => {
						const keys = Object.keys(fields)
						return keys.map((key) => {
							return (
								<Form.Item
									key={key}
									label={key}
									name={key}
									rules={[{ required: true, message: `Required ${key}!` }]}
									initialValue={fields[key]}
								>
									<Input />
								</Form.Item>
							)
						})
					})}
				<Form.Item {...tailLayout}>
					<Button type="primary" htmlType="submit">
						Submit
					</Button>
					&nbsp;&nbsp;&nbsp;
					<Link to="/backend/pages/all">
						<Button type="secendary">Cancel</Button>
					</Link>
				</Form.Item>
			</Form>
		</Card>
	)
}

export default EditPage
