import React, { useEffect } from "react"
import { FormattedMessage } from "react-intl"
import { isAdministrator } from "../../services/auth"
import * as matter from "gray-matter"
import { Link, navigate } from "gatsby"
import ProTable, { ProColumns } from "@ant-design/pro-table"
import { Button, notification, Switch } from "antd"
import moment from "moment"
import { decodeUnicode, dateFormat } from "../../utils/helper"
import { PlusOutlined } from "@ant-design/icons"

const columns: ProColumns[] = [
	{
		title: "#",
		dataIndex: "index",
		valueType: "indexBorder",
		width: 48,
		key: "key",
	},
	{
		title: "Title",
		dataIndex: "title",
		filters: true,
	},
	{
		title: "Path",
		dataIndex: "path",
		hideInSearch: true,
	},
	{
		title: "Date",
		dataIndex: "date",
		valueType: "date",
		filters: true,
	},
	{
		title: "Draft",
		dataIndex: "draft",
		filters: true,
		render: (_, record) => (
			<Switch disabled={true} defaultChecked={record.draft} size="small" />
		),
		renderFormItem: (item, { type, defaultRender, ...rest }, form) => {
			if (type === "form") {
				return null
			}
			return <Switch defaultChecked={false} />
		},
	},
	{
		title: "Options",
		valueType: "option",
		dataIndex: "path",
		align: "right",
		width: 150,
		fixed: "right",
		render: (text, row, index, action) => [
			<Link key="postEdit" to={`/backend/posts/${row.path}`}>
				<FormattedMessage id="menu.edit" />
			</Link>,
			<Link key="postView" to={`/posts/${row.path}`}>
				<FormattedMessage id="menu.view" />
			</Link>,
		],
	},
]

const Posts = () => {
	const handleGitGet = async () => {
		try {
			let appendMainData = []
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/posts/`,
				{
					method: "GET",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})
			if (fetchGitApi && fetchGitApi.length > 0) {
				for (let index = 0; index < fetchGitApi.length; index++) {
					const element = fetchGitApi[index]
					if (element.type === "dir") {
						const dirName = element.name
						const fetchGitApiPost = await fetch(
							`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/posts/${dirName}/index.md`,
							{
								method: "GET",
								headers: {
									authorization:
										"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
									"Content-Type": "application/json; charset=UTF-8",
								},
							}
						)
							.then((response) => {
								return response.json()
							})
							.catch((error) => {
								console.log("Error ", error)
								notification["error"]({
									message: "Error code " + error.status,
									description: error.statusText,
									duration: 0,
								})
							})

						if (fetchGitApiPost) {
							let appendChildData = {}
							appendChildData["path"] = dirName
							appendChildData["sha"] = fetchGitApiPost.sha
							const decodeContent: any = decodeUnicode(fetchGitApiPost.content)
							const markdownData = matter(decodeContent)
							const dataMarkdown = markdownData.data
							if (dataMarkdown) {
								const fields = Object.keys(dataMarkdown)
								fields.map((field) => {
									const fieldValue = dataMarkdown[field]
									if (field === "draft") {
										appendChildData["draft"] = fieldValue
									} else if (field === "title") {
										appendChildData["title"] = fieldValue
									} else if (field === "date") {
										appendChildData["date"] = moment(fieldValue, dateFormat)
									}
									return true
								})
							}
							appendMainData[index] = appendChildData
						}
					}
				}
				return appendMainData
			}
		} catch (error) {
			console.log("Error handleGitGet ", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: JSON.stringify(error),
				duration: 0,
			})
		}
		return false
	}

	const fetchDataSource = async (
		title: string,
		date: string,
		draft: string
	) => {
		return await handleGitGet().then((getFolder) => {
			if (Array.isArray(getFolder)) {
				const allDataSource = getFolder.map((node) => {
					return {
						key: node.sha,
						title: node.title,
						path: node.path,
						date: node.date,
						draft: node.draft,
					}
				})
				const isDraft = draft === undefined ? false : draft
				if (title || date) {
					let filterDataSource = []
					filterDataSource = allDataSource.filter(
						(f) =>
							(title && f.title.indexOf(title) > -1) ||
							(date && moment(f.date).isSame(date)) ||
							f.draft === isDraft
					)

					return filterDataSource
				} else {
					let filterDataSource = []
					filterDataSource = allDataSource.filter((f) => f.draft === isDraft)

					return filterDataSource
				}
			} else {
				return []
			}
		})
	}

	useEffect(() => {
		if (!isAdministrator()) {
			notification["warning"]({
				message: "Warning!",
				description: "Your account is not allowed!",
			})
			navigate("/backend/dashboard")
		}
	}, [])

	return (
		<ProTable
			columns={columns}
			request={async (params) => {
				const dataSource = await fetchDataSource(
					params.title,
					params.date,
					params.draft
				)
				return {
					data: dataSource,
					success: true,
				}
			}}
			rowKey="key"
			options={{
				search: false,
				fullScreen: true,
				reload: false,
				setting: true,
			}}
			pagination={{
				pageSize: 10,
				showQuickJumper: false,
				showSizeChanger: true,
			}}
			dateFormatter="string"
			toolBarRender={() => [
				<Link to="/backend/posts/new">
					<Button key="button" icon={<PlusOutlined />} type="primary">
						<FormattedMessage id="menu.new.post" />
					</Button>
				</Link>,
			]}
		/>
	)
}

export default Posts
