import React, { useEffect, useState } from "react"
import { isAdministrator } from "../../../services/auth"
import { encodeUnicode, decodeUnicode, dateFormat } from "../../../utils/helper"
import * as matter from "gray-matter"
import { Link, navigate } from "gatsby"
import {
	Card,
	Form,
	Input,
	Button,
	notification,
	Upload,
	Image,
	Spin,
	DatePicker,
	Switch,
	Empty,
} from "antd"
import moment from "moment"
import { UploadOutlined } from "@ant-design/icons"

const { TextArea } = Input
const layout = {
	labelCol: { span: 6 },
	wrapperCol: { span: 17 },
}
const tailLayout = {
	wrapperCol: { offset: 6, span: 17 },
}

const EditPost = ({ slug }) => {
	const [sha, setSha] = useState("")
	const [shaFeatured, setShaFeatured] = useState("")
	const [featuredUrl, setFeaturedUrl] = useState("")
	const [fileFeatured, setFileFeatured] = useState([])
	const [form] = Form.useForm()
	const [uploading, setUploading] = useState(false)

	const onFinish = (values) => {
		let contentStr = "---\n"
		let htmlMarkdown = ""
		if (values) {
			for (const [key, value] of Object.entries(values)) {
				if (key === "markdownContent") {
					htmlMarkdown = value
				} else if (key === "date") {
					const dateValue = moment(value).format(dateFormat)
					contentStr += key + `: ` + dateValue + `\n`
				} else {
					contentStr += key + `: ` + value + `\n`
				}
			}
		}
		contentStr += "---\n\n"
		contentStr += htmlMarkdown
		handleGitUpdate(contentStr, sha).then((updateFile) => {
			if (updateFile) {
				notification["success"]({
					message: "Success!",
					description:
						"Updated markdown/posts/" + slug + "/index.md successfully.",
				})
			} else {
				let errMsg = "No status"
				if (updateFile === 404) {
					errMsg = "Status: 404 Not Found"
				} else if (updateFile === 409) {
					errMsg = "Status: 409 Conflict"
				} else if (updateFile === 422) {
					errMsg = "Status: 422 Unprocessable Entity"
				}
				notification["error"]({
					message: "Error!",
					description: errMsg,
					duration: 0,
				})
			}
		})
	}
	const handleGitGet = async () => {
		try {
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/posts/${slug}/index.md`,
				{
					method: "GET",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return fetchGitApi
			} else {
				notification["info"]({
					message: "Empty Fetch",
				})
			}
		} catch (error) {
			notification["error"]({
				message: "Error handleGitGet!",
				description: JSON.stringify(error),
				duration: 0,
			})
		}
		return false
	}
	const handleGitGetFeatured = async () => {
		setUploading(true)
		try {
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/posts/${slug}/featured.jpg`,
				{
					method: "GET",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return fetchGitApi
			} else {
				notification["info"]({
					message: "Empty Fetch",
				})
			}
		} catch (error) {
			setUploading(false)
			console.log("Error handleGitGetFeatured ", error)
			notification["error"]({
				message: "Error handleGitGetFeatured!",
				description: JSON.stringify(error),
				duration: 0,
			})
		}
		return false
	}
	const handleGitUpdateFeatured = async (contentStr) => {
		try {
			const data = {
				message:
					"[skip ci] Update markdown/posts/" +
					slug +
					"/featured.jpg from Modern backend.",
				content: btoa(contentStr),
				sha: shaFeatured,
			}
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/posts/${slug}/featured.jpg`,
				{
					method: "PUT",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
					body: JSON.stringify(data),
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return fetchGitApi
			} else {
				notification["info"]({
					message: "Empty Fetch",
				})
			}
		} catch (error) {
			console.log("Error handleGitUpdateFeatured ", error)
			notification["error"]({
				message: "Error handleGitUpdateFeatured!",
				description: JSON.stringify(error),
				duration: 0,
			})
		}
		return false
	}

	const handleBeforeUpload = (file) => {
		setUploading(true)
		const reader = new FileReader()
		reader.readAsBinaryString(file)
		reader.onload = () => {
			handleGitUpdateFeatured(reader.result).then((getFeatured) => {
				if (getFeatured !== false) {
					const featuredFile = [
						{
							uid: getFeatured.content.sha,
							name: getFeatured.content.name,
							status: "done",
							url: getFeatured.content.download_url,
						},
					]
					setFileFeatured(featuredFile)
					setShaFeatured(getFeatured.content.sha)
					setFeaturedUrl(getFeatured.content.download_url)
					setUploading(false)
					notification["success"]({
						message: "Success!",
						description:
							"Upload featured image successfully, No need to submit form!",
					})
				} else {
					notification["error"]({
						message: "Error!",
						description: "Upload featured image failed, Please try again!",
						duration: 0,
					})
				}
			})
		}
		return false
	}
	const handleGitUpdate = async (contentStr, fileSha) => {
		try {
			const data = {
				message:
					"[skip ci] Update markdown/posts/" +
					slug +
					"/index.md from Modern backend.",
				content: encodeUnicode(contentStr),
				sha: fileSha,
			}
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/posts/${slug}/index.md`,
				{
					method: "PUT",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
					body: JSON.stringify(data),
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return true
			} else {
				notification["info"]({
					message: "Empty Fetch",
				})
			}
		} catch (error) {
			console.log("Error handleGitUpdate!", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: JSON.stringify(error),
				duration: 0,
			})
		}
		return false
	}

	useEffect(() => {
		if (!isAdministrator()) {
			notification["warning"]({
				message: "Warning!",
				description: "Your account is not allowed!",
			})
			navigate("/backend/dashboard")
		}

		async function fetchOauth() {
			await handleGitGet().then((getFile) => {
				if (getFile !== false) {
					setSha(getFile.sha)
					const decodeContent = decodeUnicode(getFile.content)
					const markdownData = matter(decodeContent)
					const dataMarkdown = markdownData.data
					if (dataMarkdown) {
						const fields = Object.keys(dataMarkdown)
						fields.map((field) => {
							const fieldValue = dataMarkdown[field]
							if (field === "draft") {
								form.setFieldsValue({
									draft: fieldValue,
								})
							} else if (field === "title") {
								form.setFieldsValue({
									title: fieldValue,
								})
							} else if (field === "description") {
								form.setFieldsValue({
									description: fieldValue,
								})
							} else if (field === "date") {
								form.setFieldsValue({
									date: moment(fieldValue, dateFormat),
								})
							} else if (field === "featured") {
								form.setFieldsValue({
									featured: fieldValue,
								})
							}
							return true
						})
					}
					const contentMarkdown = markdownData.content
					form.setFieldsValue({
						markdownContent: contentMarkdown,
					})
				}
			})
			await handleGitGetFeatured().then((getFeatured) => {
				if (getFeatured !== false) {
					setUploading(true)
					const featuredFile = [
						{
							uid: getFeatured.sha,
							name: getFeatured.name,
							status: "done",
							url: getFeatured.download_url,
						},
					]
					setFileFeatured(featuredFile)
					setShaFeatured(getFeatured.sha)
					setFeaturedUrl(getFeatured.download_url)
					setUploading(false)
				}
			})
		}
		fetchOauth()
		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, [])

	return (
		<Card>
			<Form
				{...layout}
				name="editForm"
				form={form}
				onFinish={onFinish}
				className="customForm"
			>
				<Form.Item label="Draft" name="draft" valuePropName="checked">
					<Switch />
				</Form.Item>
				<Form.Item
					label="Title"
					name="title"
					rules={[{ required: true, message: "Please input Title!" }]}
				>
					<Input />
				</Form.Item>
				<Form.Item
					label="Description"
					name="description"
					rules={[{ required: true, message: "Please input Description!" }]}
				>
					<Input />
				</Form.Item>
				<Form.Item
					label="Date"
					name="date"
					rules={[{ required: true, message: "Please input Date!" }]}
				>
					<DatePicker format={dateFormat} />
				</Form.Item>
				<Form.Item
					label="Featured"
					name="featured"
					rules={[{ required: true, message: "Missing Featured name!" }]}
					hidden={true}
				>
					<Input />
				</Form.Item>
				<Form.Item label="Featured image">
					<div>
						{uploading ? (
							<Spin />
						) : featuredUrl && featuredUrl.length > 0 ? (
							<Image width={250} src={featuredUrl} />
						) : (
							<Empty image={Empty.PRESENTED_IMAGE_SIMPLE} />
						)}
					</div>
				</Form.Item>
				<Form.Item {...tailLayout}>
					<Upload
						accept="image/jpeg"
						multiple={false}
						listType="picture"
						showUploadList={false}
						fileList={fileFeatured}
						beforeUpload={handleBeforeUpload}
					>
						<Button icon={<UploadOutlined />} loading={uploading}>
							Upload
						</Button>
					</Upload>
				</Form.Item>
				<Form.Item
					label="Markdown content"
					name="markdownContent"
					rules={[
						{ required: true, message: "Please input Markdown content!" },
					]}
				>
					<TextArea autoSize={{ minRows: 3 }} />
				</Form.Item>
				<Form.Item {...tailLayout}>
					<Button type="primary" htmlType="submit">
						Submit
					</Button>
					&nbsp;&nbsp;&nbsp;
					<Link to="/backend/posts/all">
						<Button type="secendary">Cancel</Button>
					</Link>
				</Form.Item>
			</Form>
		</Card>
	)
}

export default EditPost
