import React, { useEffect, useState } from "react"
import { isAdministrator } from "../../../services/auth"
import { encodeUnicode, today, dateFormat } from "../../../utils/helper"
import { Link, navigate } from "gatsby"
import {
	Card,
	Form,
	Input,
	Button,
	notification,
	Upload,
	Image,
	Spin,
	message,
	DatePicker,
	Switch,
} from "antd"
import moment from "moment"
import { UploadOutlined } from "@ant-design/icons"

const { TextArea } = Input
const layout = {
	labelCol: { span: 6 },
	wrapperCol: { span: 17 },
}
const tailLayout = {
	wrapperCol: { offset: 6, span: 17 },
}

const NewPost = () => {
	const [featuredUrl, setFeaturedUrl] = useState("")
	const [fileFeatured, setFileFeatured] = useState([])
	const [form] = Form.useForm()
	const [uploading, setUploading] = useState(false)
	const [canUpload, setCanUpload] = useState(true)

	const onFinish = (values) => {
		let contentStr = "---\n"
		let htmlMarkdown = ""
		let slug = "please-change"
		if (values) {
			for (const [key, value] of Object.entries(values)) {
				if (key === "markdownContent") {
					htmlMarkdown = value
				} else {
					if (key === "slug") {
						slug = value
					} else if (key === "date") {
						const dateValue = moment(value).format(dateFormat)
						contentStr += key + `: ` + dateValue + `\n`
					} else {
						contentStr += key + `: ` + value + `\n`
					}
				}
			}
		}
		contentStr += "---\n\n"
		contentStr += htmlMarkdown
		handleGitAdd(contentStr, slug).then((addFile) => {
			if (addFile) {
				notification["success"]({
					message: "Success!",
					description:
						"Added markdown/posts/" + slug + "/index.md successfully.",
				})
				navigate("/backend/posts/all")
			} else {
				let errMsg = "No status"
				if (addFile === 404) {
					errMsg = "Status: 404 Not Found"
				} else if (addFile === 409) {
					errMsg = "Status: 409 Conflict"
				} else if (addFile === 422) {
					errMsg = "Status: 422 Unprocessable Entity"
				}
				notification["error"]({
					message: "Error!",
					description: errMsg,
					duration: 0,
				})
			}
		})
	}
	const onSlugChanged = (e) => {
		const slugExist = e.target.value ? true : false
		setCanUpload(slugExist)
	}
	const handleGitAddFeatured = async (contentStr) => {
		const slug = form.getFieldValue("slug")
			? form.getFieldValue("slug")
			: "please-change"

		try {
			const data = {
				message:
					"[skip ci] Added markdown/posts/" +
					slug +
					"/featured.jpg from Modern backend.",
				content: btoa(contentStr),
			}
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/posts/${slug}/featured.jpg`,
				{
					method: "PUT",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
					body: JSON.stringify(data),
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return fetchGitApi
			} else {
				notification["info"]({
					message: "Empty Fetch",
				})
			}
		} catch (error) {
			console.log("Error handleGitAddFeatured ", error)
			notification["error"]({
				message: "Error handleGitAddFeatured!",
				description: JSON.stringify(error),
				duration: 0,
			})
		}
		return false
	}

	const handleBeforeUpload = (file) => {
		if (file.type !== "image/jpeg") {
			message.error(`${file.name} is not a JPG or JPEG file`)
			return false
		}

		setUploading(true)
		setCanUpload(false)
		const reader = new FileReader()
		reader.readAsBinaryString(file)
		reader.onload = () => {
			handleGitAddFeatured(reader.result).then((getFeatured) => {
				if (getFeatured !== false) {
					const featuredFile = [
						{
							uid: getFeatured.content.sha,
							name: getFeatured.content.name,
							status: "done",
							url: getFeatured.content.download_url,
						},
					]
					setFileFeatured(featuredFile)
					setFeaturedUrl(getFeatured.content.download_url)
					setUploading(false)
					setCanUpload(true)
					notification["success"]({
						message: "Success!",
						description: "Upload featured image successfully!",
					})
				} else {
					notification["error"]({
						message: "Error!",
						description: "Upload featured image failed, Please try again!",
						duration: 0,
					})
				}
			})
		}
		return false
	}
	const handleGitAdd = async (contentStr, slug) => {
		try {
			const data = {
				message:
					"[skip ci] Added markdown/posts/" +
					slug +
					"/index.md from Modern backend.",
				content: encodeUnicode(contentStr),
			}
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/markdown/posts/${slug}/index.md`,
				{
					method: "PUT",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
					body: JSON.stringify(data),
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return true
			} else {
				notification["info"]({
					message: "Empty Fetch",
				})
			}
		} catch (error) {
			console.log("Error handleGitAdd!", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: JSON.stringify(error),
				duration: 0,
			})
		}
		return false
	}

	useEffect(() => {
		if (!isAdministrator()) {
			notification["warning"]({
				message: "Warning!",
				description: "Your account is not allowed!",
			})
			navigate("/backend/dashboard")
		}
	}, [])

	return (
		<Card>
			<Form
				{...layout}
				name="addForm"
				form={form}
				initialValues={{
					draft: false,
					slug: "please-change",
					date: moment(today, dateFormat),
					featured: "featured.jpg",
				}}
				onFinish={onFinish}
				className="customForm"
			>
				<Form.Item label="Draft" name="draft">
					<Switch defaultChecked={false} />
				</Form.Item>
				<Form.Item
					label="Slug (Folder name)"
					name="slug"
					rules={[{ required: true, message: "Please input slug!" }]}
					onChange={onSlugChanged}
				>
					<Input />
				</Form.Item>
				<Form.Item
					label="Title"
					name="title"
					rules={[{ required: true, message: "Please input Title!" }]}
				>
					<Input />
				</Form.Item>
				<Form.Item
					label="Description"
					name="description"
					rules={[{ required: true, message: "Please input Description!" }]}
				>
					<Input />
				</Form.Item>
				<Form.Item
					label="Date"
					name="date"
					rules={[{ required: true, message: "Please input Date!" }]}
				>
					<DatePicker format={dateFormat} />
				</Form.Item>
				<Form.Item
					label="Featured"
					name="featured"
					rules={[{ required: true, message: "Please input Featured!" }]}
					hidden={true}
				>
					<Input />
				</Form.Item>
				{featuredUrl ? (
					<Form.Item label="Featured image">
						<div>
							{uploading ? <Spin /> : <Image width={250} src={featuredUrl} />}
						</div>
					</Form.Item>
				) : null}
				<Form.Item label="Featured image">
					<Upload
						accept="image/jpeg"
						multiple={false}
						listType="picture"
						showUploadList={false}
						fileList={fileFeatured}
						beforeUpload={handleBeforeUpload}
					>
						<Button
							icon={<UploadOutlined />}
							loading={uploading}
							disabled={!canUpload}
						>
							*JPG or JPEG Only!
						</Button>
					</Upload>
				</Form.Item>
				<Form.Item
					label="Markdown content"
					name="markdownContent"
					rules={[
						{ required: true, message: "Please input Markdown content!" },
					]}
				>
					<TextArea autoSize={{ minRows: 3 }} />
				</Form.Item>
				<Form.Item {...tailLayout}>
					<Button type="primary" htmlType="submit">
						Submit
					</Button>
					&nbsp;&nbsp;&nbsp;
					<Link to="/backend/posts/all">
						<Button type="secendary">Cancel</Button>
					</Link>
				</Form.Item>
			</Form>
		</Card>
	)
}

export default NewPost
